var express    = require('express');
var bodyParser = require('body-parser');

var exec       = require('child_process').exec;
var execSync   = require('child_process').execSync;
var fs         = require('fs');
var path       = require('path');
var fsReaddir = require('fs-readdir');
var Zip        = require('node-zip');
var JSZip = require('jszip');

var exphbs = require('express-handlebars');

//initializer
var initGitRepos = require('./initializers/gitRepos.js');
//load names of available download files
const downloadFiles = require('./downloadFiles.json');
const downloadNames = Object.keys(downloadFiles);

//load functions for frontend and backend build
let backendFunctions = require('./backendFunctions.js');
let frontendFunctions = require('./frontendFunctions.js');

var app = express();

app.use(bodyParser.urlencoded({
	extended: true
}));
app.use(bodyParser.json());

	//set handlebars as view engine
	app.engine('.hbs', exphbs({extname: '.hbs'}));
	app.set('view engine', '.hbs');

	
	//TODO add spinner for telling user to wait until initialization is done
	
	//initGitRepos.initialize(downloadFiles);

	//routes
	app.get('/', function(req, res){
		res.render('home', {downloadFiles: downloadNames});
	});


	app.post('/', function(req, res, next){
		const checkedFiles = Object.keys(req.body);

		if(typeof checkedFiles !== 'undefined' && checkedFiles.length > 0){
			res.render('download', {checkedFiles: checkedFiles});
		}
		else {
			res.render('home', {downloadFiles: downloadNames});
		}
	});


	app.get('/download', function(req, res){
		//holds promises for backend and/or frontend
		const promiseArray = []
		const checkedFiles = req.query.checkedFiles.split(',');



		backendChecked = checkedFiles.find(function(file){
			return file === 'explorviz-ui-backend';
		});

		if (backendChecked){

			const fileName = backendChecked;

			function backendAll(){
				return backendFunctions.backendClean(fileName).then(function(){
					return backendFunctions.backendPull(fileName).then(function(){
						return backendFunctions.backendInstallAddons(checkedFiles,downloadFiles).then(function(){
							return backendFunctions.backendBuild(fileName)
						})
					})
				}).catch(function(error){
					let response = 
					'There was an error during your backend build. ' +
					'Please contact "explorviz-developers-request@listserv.dfn.de" ' +
					'and add the following information: ' + error;
					res.send(response);

				})

			}
			promiseArray.push(backendAll());

		} 

		frontendChecked = checkedFiles.find(function(file){
			return file === 'explorviz-ui-frontend';
		});

		if (frontendChecked){
			const fileName = frontendChecked;
			const downloadURL = downloadFiles[frontendChecked]; 

			function frontendAll() {
				return frontendFunctions.frontendClean(fileName).then(function(){
					return frontendFunctions.frontendPull(fileName).then(function(){
						return frontendFunctions.frontendInstallAddons(fileName, checkedFiles, downloadFiles).then(function(){
							return frontendFunctions.frontendBuild(fileName)
						})
					})
				}).catch(function(error){
					let response = 
					'There was an error during your frontend build. ' +
					'Please contact "explorviz-developers-request@listserv.dfn.de" ' +
					'and add the following information: ' + error;
					res.send(response);

				})
			}
			promiseArray.push(frontendAll());

		};

		finalizeZip();
//TODO put all zip-things in zipFunctions.js;
//problem: createZip has to know the res from /download-route and if just the other functions will be exported, the zip has to be passed to createZip with resolve()
// create a file to stream archive data to. 
const zip = new JSZip();
zip.file("README.txt", "text for README");
let finishedArray=[];

function streamFilesToZip(){
	
	return new Promise((resolve,reject)=> {

		Promise.all(promiseArray).then(files => {
						
			files.forEach(file => {
				if (file.zipFileName === 'backend.war'){

					fs.readFileAsync = function(filePath){
						return new Promise(function (resolve, reject) {
							fs.readFile(filePath, function(err, data) {
								if (err) {
									reject(err);
								} else {
									resolve(data);
								}

							});
						});
					}

					function addBackendToZip(data){
						return new Promise(function(resolve,reject){
							zip.file(file.zipFileName, data);
							resolve();
						})
					}
					console.log('in backend.war');

					backendContent = fs.readFileAsync(file.path);
					backendContent.then(function(data){
						addBackendToZip(data).then(function(){
							finishedArray.push('backend finished');
							if(finishedArray.length === promiseArray.length){
								resolve(zip);
							}

						})

					})

				}
				if(file.zipFileName === 'frontend'){
					const frontendStream = fsReaddir(file.path)
					.on('error', function(err) {
						console.log('error:', err);
					}).on('data', function(data){
						for (let i=0; i<data.length; i++) {
							const frontendReadStream = fs.createReadStream(data[i]);
							zip.file(data[i], frontendReadStream);
						}

					}).on('finish', function(){
						finishedArray.push('frontend finished');
						if(finishedArray.length === promiseArray.length){
							resolve(zip);
						}
						console.log('finishedArray in frontend:', finishedArray);


					})



				}



			})


		})

	})
}


function finalizeZip(){
	return streamFilesToZip().then(function(zip){
		return createZip(zip)
	}).catch(function(error) {let response = 
		'There was an error during your build. ' +
		'Please contact "explorviz-developers-request@listserv.dfn.de" ' +
		'and add the following information: ' + error;
		res.send(response);
	})

}

function createZip(zip){
	console.log('in createZip');
	return new Promise((resolve,reject) => {
		zip.generateNodeStream({type:'nodebuffer',streamFiles:true}).pipe(fs.createWriteStream('explorviz-builds.zip')).on('error', function(error){
			console.log('error in createZip:', error);
		}).on('finish', function () {
			console.log('explorviz-builds.zip written.');
			res.download('explorviz-builds.zip');
		}); 

	})

	
}


});


app.listen(3000);
console.log("Downloader started on port 3000.");